@@ -191,7 +191,7 @@ class BaseUtilsMixin:
         e.g. 1-3 , will get 1 to 3.
         """
         ids = []
-        rex = re.compile("^(?P<begin>\d+)(?:-(?P<end>\d+))?$")
+        rex = re.compile("^(?P<begin>\\d+)(?:-(?P<end>\\d+))?$")
         rex = rex.match(args)
         if hasattr(rex, 'groupdict'):
             try:
@@ -232,14 +232,14 @@ class BaseUtilsMixin:
     def _format_line(self, tag_pad, nid="ID", user="USER", url="URL",
                      tags="TAGS"):
 
-        user_pad = int(self.config.get_value("UI", "URL_pad"))
-        url_pad = int(self.config.get_value("UI", "user_pad"))
-        tag_pad = int(self.config.get_value("UI", "tag_pad"))
+        user_pad = int(self.config.get_value("UI", "URL_pad").split('#')[0])
+        url_pad = int(self.config.get_value("UI", "user_pad").split('#')[0])
+        tag_pad = int(self.config.get_value("UI", "tag_pad").split('#')[0])
+        line = self.config.get_value("UI", "line_format")
 
-        return ("{ID:<3} {USER:<{us}}{URL:<{ur}}{Tags:<{tg}}"
-                "".format(ID=nid, USER=user,
-                          URL=url, Tags=tags, us=user_pad,
-                          ur=url_pad, tg=tag_pad))
+        return (line.format(ID=nid, USER=user,
+                            URL=url, Tags=tags, user_pad=user_pad,
+                            url_pad=url_pad, tag_pad=tag_pad))
 
     def _print_node_line(self, node, rows, cols, url_filter):
         if url_filter != "" and node.url.find(url_filter) == -1:
@@ -247,8 +247,9 @@ class BaseUtilsMixin:
 
         tagstring = ','.join([t.decode() for t in node.tags])
 
-        if len(node.url) > int(self.config.get_value("UI", "URL_length")):
-            node_url = node.url[:int(self.config.get_value("UI", "URL_length"))] + "..." # noqa
+        if len(node.url) > int(self.config.get_value(
+                "UI", "URL_length").split("#")[0]):
+            node_url = node.url[:int(self.config.get_value("UI", "URL_length").split("#")[0])] + "..." # noqa
         else:
             node_url = node.url
 
@@ -264,21 +265,26 @@ class BaseUtilsMixin:
             filter = args.split()[0]
             ce = CryptoEngine.get()
             filter = ce.encrypt(filter)
-        nodeids = self._db.listnodes(filter=filter)
-        return nodeids
+        return self._db.listnodes(filter=filter)
 
-    def _db_entries_to_nodes(self, raw_nodes):
-        _nodes_inst = []
+    def _lazy_get_node_ids(self, args):
+        filter = None
+        if args:
+            filter = args.split()[0]
+            ce = CryptoEngine.get()
+            filter = ce.encrypt(filter)
+        for node_id in self._db.lazy_list_node_ids(filter=filter):
+            yield node_id
+
+    def _db_entry_to_node(self, raw_node):
         # user, pass, url, notes
-        for node in raw_nodes:
-            _nodes_inst.append(Node.from_encrypted_entries(
-                node[1],
-                node[2],
-                node[3],
-                node[4],
-                node[5:]))
-            _nodes_inst[-1]._id = node[0]
-        return _nodes_inst
+        node_inst = Node.from_encrypted_entries(raw_node[1],
+                                                raw_node[2],
+                                                raw_node[3],
+                                                raw_node[4],
+                                                raw_node[5:])
+        node_inst._id = raw_node[0]
+        return node_inst
 
     def _get_input(self, prompt):
         print(prompt, end="")
@@ -313,12 +319,12 @@ class BaseUtilsMixin:
             print("print accepts only a single ID ...")
             return
 
-        nodes = self._db.getnodes([nodeid])
-        if not nodes:  # pragma: no cover
+        node = self._db.get_node(nodeid)
+        if not node:  # pragma: no cover
             print("Node not found ...")
             return
 
-        node = self._db_entries_to_nodes(nodes)[0]
+        node = self._db_entry_to_node(node)
         return node
 
 
@@ -339,28 +345,34 @@ class BaseCommands(HelpUIMixin, AliasesMixin, BaseUtilsMixin):
             print("Error: {0} ".format(exception))
 
     def do_copy(self, args):  # pragma: no cover
-        """copy item to clipboard"""
+        """copy item to clipboard. Accepts an id or `u:<URL substring>`"""
         if not self.hasxsel:
             return
-        if not args.isdigit():
-            print("Copy accepts only IDs ...")
-            return
-        ids = args.split()
-        if len(ids) > 1:
-            print("Can copy only 1 password at a time...")
+        if not args.isdigit() and not args.startswith("u:"):
             return
 
-        ce = CryptoEngine.get()
+        url_filter = ""
+        m = re.search('^u:([^ ]+) ?(.*)$', args)
+
+        if m:
+            url_filter, args = m.groups()
+            ids = self._db.lazy_list_node_ids()
+        else:
+            ids = args.split()
+            if len(ids) > 1:
+                print("Can copy only 1 password at a time...")
+                return
+
         nodes = self._db.getnodes(ids)
 
+        ce = CryptoEngine.get()
         for node in nodes:
-            password = ce.decrypt(node[2])
-            tools.text_to_clipboards(password)
-            flushtimeout = self.config.get_value('Global', 'cp_timeout')
-            flushtimeout = flushtimeout or 10
-            print("erasing in {} sec...".format(flushtimeout))
-            time.sleep(int(flushtimeout))
-            tools.text_to_clipboards("")
+            if ce.decrypt(node[3]).find(url_filter.encode()) != -1:
+                password = ce.decrypt(node[2])
+                tools.text_to_clipboards(password)
+                flushtimeout = self.config.get_value('Global', 'cp_timeout')
+                flushtimeout = flushtimeout or 10
+                print("erasing in {} sec...".format(flushtimeout))
 
     def do_open(self, args):  # pragma: no cover
         ids = self._get_ids(args)
@@ -397,8 +409,7 @@ class BaseCommands(HelpUIMixin, AliasesMixin, BaseUtilsMixin):
 
         filename = args.get('filename', 'pwman-export.csv')
         delim = args.get('delimiter', ';')
-        nodeids = self._db.listnodes()
-        nodes = self._db.getnodes(nodeids)
+        nodes = self._db.getnodes(list(self._db.lazy_list_node_ids()))
 
         with open(filename, 'w') as csvfile:
             writer = csv.writer(csvfile, delimiter=delim)
@@ -456,11 +467,10 @@ class BaseCommands(HelpUIMixin, AliasesMixin, BaseUtilsMixin):
         ids = self._get_ids(args)
         for i in ids:
             i = int(i)
-            node = self._db.getnodes([i])
+            node = self._db.get_node(i)
             if not node:
                 print("Node not found ...")
                 return
-            node = node[0]
             node = node[1:5] + [node[5:]]
             node = Node.from_encrypted_entries(*node)
             if not menu:
@@ -488,14 +498,12 @@ class BaseCommands(HelpUIMixin, AliasesMixin, BaseUtilsMixin):
         if m:
             url_filter, args = m.groups()
 
-        nodeids = self._get_node_ids(args)
-        raw_nodes = self._db.getnodes(nodeids)
-        _nodes_inst = self._db_entries_to_nodes(raw_nodes)
+        nodeids_gen = self._lazy_get_node_ids(args)
         head = self._format_line(cols - 32)
         print(tools.typeset(head, Fore.YELLOW, False))
-
-        for idx, node in enumerate(_nodes_inst):
-            self._print_node_line(node, rows, cols, url_filter)
+        for node in self._db.getnodes(nodeids_gen):
+            self._print_node_line(self._db_entry_to_node(node), rows, cols,
+                                  url_filter)
 
     def do_new(self, args):  # pragma: no cover
         # The cmd module stops if any of do_* return something
@@ -506,13 +514,11 @@ class BaseCommands(HelpUIMixin, AliasesMixin, BaseUtilsMixin):
         print(node.id)
 
     def do_pp(self, args):
-
+        """print the password only"""
         node = self._get_node(args)
-
         print(node.password)
 
     def do_print(self, args):
-
         node = self._get_node(args)
 
         print(node)
